@@ -354,6 +354,7 @@ main(int argc, char *argv[])
                    _("failed to release tty properly"));
             kill(getppid(), SIGHUP);
         }
+        dav_set_no_terminal();
     }
 
     if (!ret) {
@@ -681,7 +682,7 @@ check_fstab(const dav_args *args)
 
     setfsent();
     struct fstab *ft = getfsfile(mpoint);
-    if (!ft)
+    if (!ft || !ft->fs_spec)
         error(EXIT_FAILURE, 0, _("no entry for %s found in %s"), url,
               _PATH_FSTAB);
 
@@ -695,11 +696,20 @@ check_fstab(const dav_args *args)
     if (ft->fs_mntops)
         get_options(n_args, ft->fs_mntops);
 
-    if (strcmp(DAV_FS_TYPE, ft->fs_vfstype) != 0)
+    if (! ft->fs_vfstype || strcmp(DAV_FS_TYPE, ft->fs_vfstype) != 0)
         error(EXIT_FAILURE, 0, _("different file system type in %s"),
               _PATH_FSTAB);
-    if (strcmp(args->conf, n_args->conf) != 0)
-        error(EXIT_FAILURE, 0, _("different config file in %s"), _PATH_FSTAB);
+    if (args->conf || n_args->conf) {
+        if (!args->conf || !n_args->conf
+                || strcmp(args->conf, n_args->conf) != 0)
+            error(EXIT_FAILURE, 0, _("different config file in %s"),
+                  _PATH_FSTAB);
+    }
+    if (args->cl_username || n_args->cl_username) {
+        if (!args->cl_username || !n_args->cl_username
+                || strcmp(args->cl_username, n_args->cl_username) != 0)
+            error(EXIT_FAILURE, 0, _("different username in %s"), _PATH_FSTAB);
+    }
     if (!n_args->user && !n_args->users)
         error(EXIT_FAILURE, 0,
               _("neither option `user' nor option `users' set in %s"),
@@ -1140,6 +1150,17 @@ parse_secrets(dav_args *args)
         read_secrets(args, args->secrets);
     }
 
+    if (args->cl_username) {
+        if (args->username)
+            free(args->username);
+        args->username = args->cl_username;
+        args->cl_username = NULL;
+        if (args->password)
+            free(args->password);
+        args->password = NULL;
+        args->p_passwd = user_input(_("Password: "));
+    }
+
     if (args->askauth && args->useproxy && !args->p_user) {
         printf(_("Please enter the username to authenticate with proxy\n"
                  "%s or hit enter for none.\n"), args->p_host);
@@ -1149,7 +1170,11 @@ parse_secrets(dav_args *args)
     if (args->askauth && args->useproxy && args->p_user && !args->p_passwd) {
         printf(_("Please enter the password to authenticate user %s with proxy\n"
                  "%s or hit enter for none.\n"), args->p_user, args->p_host);
-        args->p_passwd = dav_user_input_hidden(_("Password: "));
+        if (isatty(fileno(stdin))) {
+            args->p_passwd = dav_user_input_hidden(_("Password: "));
+        } else {
+            args->p_passwd = user_input(_("Password: "));
+        }
         if (args->p_passwd && strlen(args->p_passwd) == 0) {
             free(args->p_passwd);
             args->p_passwd = NULL;
@@ -1165,7 +1190,11 @@ parse_secrets(dav_args *args)
     if (args->askauth && args->username && !args->password) {
         printf(_("Please enter the password to authenticate user %s with "
                  "server\n%s or hit enter for none.\n"), args->username, url);
-        args->password = dav_user_input_hidden(_("Password: "));
+        if (isatty(fileno(stdin))) {
+            args->password = dav_user_input_hidden(_("Password: "));
+        } else {
+            args->password = user_input(_("Password: "));
+        }
         if (args->password && strlen(args->password) == 0) {
             free(args->password);
             args->password = NULL;
@@ -1176,6 +1205,8 @@ parse_secrets(dav_args *args)
         syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG), "Secrets:");
         syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
                "  username: %s", args->username);
+        syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
+               "  cl_username: %s", args->cl_username);
         syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
                "  password: %s", args->password);
         syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_DEBUG),
@@ -1409,6 +1440,8 @@ delete_args(dav_args *args)
         memset(args->username, '\0', strlen(args->username));
         free(args->username);
     }
+    if (args->cl_username)
+        free(args->cl_username);
     if (args->password) {
         memset(args->password, '\0', strlen(args->password));
         free(args->password);
@@ -1494,6 +1527,7 @@ get_options(dav_args *args, char *option)
 {
     enum {
         CONF = 0,
+        USERNAME,
         UID,
         GID,
         FILE_MODE,
@@ -1519,6 +1553,7 @@ get_options(dav_args *args, char *option)
     };
     char *suboptions[] = {
         [CONF] = "conf",
+        [USERNAME] = "username",
         [UID] = "uid",
         [GID] = "gid",
         [FILE_MODE] = "file_mode",
@@ -1560,6 +1595,11 @@ get_options(dav_args *args, char *option)
                 free(args->conf);
             args->conf = ne_strdup(argument);
             break;
+        case USERNAME:
+            if (args->cl_username)
+                free(args->cl_username);
+            args->cl_username = ne_strdup(argument);
+            break;
         case UID:
             pwd = getpwnam(argument);
             if (!pwd) {
@@ -1727,6 +1767,7 @@ new_args(void)
         args->secrets = NULL;
     }
     args->username = NULL;
+    args->cl_username = NULL;
     args->password = NULL;
     args->clicert = NULL;
     args->clicert_pw = NULL;
