@@ -193,7 +193,7 @@ static int initialized;
 /* Whether a terminal is available to communicate with the user.
    Should be reset with set_no_terminal() when forking into daemon mode.
    Needed by  ssl_verify() which may be called at any time. */
-static int have_terminal = 1;
+static int have_terminal;
 
 /* Handle to convert character encoding from utf-8 to LC_CTYPE.
    If NULL no conversion is done. */
@@ -370,7 +370,11 @@ dav_init_webdav(const dav_args *args)
         username = ne_strdup(args->username);
     if (args->password)
         password = ne_strdup(args->password);
+#if NE_VERSION_MINOR < 26
     ne_set_server_auth(session, auth, "server");
+#else /* NE_VERSION_MINOR >= 26 */
+    ne_add_server_auth(session, NE_AUTH_ALL, auth, "server");
+#endif /* NE_VERSION_MINOR >= 26 */
 
     if (args->useproxy && args->p_host) {
         ne_session_proxy(session, args->p_host, args->p_port);
@@ -378,7 +382,11 @@ dav_init_webdav(const dav_args *args)
             p_username = ne_strdup(args->p_user);
         if (args->p_passwd)
             p_password = ne_strdup(args->p_passwd);
+#if NE_VERSION_MINOR < 26
         ne_set_proxy_auth(session, auth, "proxy");
+#else /* NE_VERSION_MINOR >= 26 */
+        ne_add_proxy_auth(session, NE_AUTH_ALL, auth, "proxy");
+#endif /* NE_VERSION_MINOR >= 26 */
     }
 
     if (strcmp(args->scheme, "https") == 0) {
@@ -431,6 +439,8 @@ dav_init_webdav(const dav_args *args)
         }
     }
 
+    have_terminal = args->askauth;
+
     if (args->locks) {
         locks = ne_lockstore_create();
         if (!args->lock_owner) {
@@ -769,8 +779,11 @@ dav_head(const char *path, char **etag, time_t *mtime, off_t *length,
     }
 
     value = ne_get_response_header(req, "Last-Modified");
-    if (!ret && mtime && value)
-        *mtime = ne_httpdate_parse(value);
+    if (!ret && mtime && value) {
+        time_t lm = ne_httpdate_parse(value);
+        if (lm)
+            *mtime = lm;
+    }
 
     value = ne_get_response_header(req, "Content-Length");
     if (!ret && length && value)
@@ -1685,9 +1698,8 @@ prop_result(void *userdata, const char *href, const ne_prop_result_set *set)
         return;
     }
 
-    dav_props *result = ne_calloc(sizeof(dav_props));
-    result->path = ne_path_unescape(uri.path);
-    ne_uri_free(&uri);
+    char *tmp_path = (char *) ne_malloc(strlen(uri.path) + 1);
+    const char *from = uri.path;
 
 #else /* NE_VERSION_MINOR >= 26 */
 
@@ -1698,11 +1710,27 @@ prop_result(void *userdata, const ne_uri *uri, const ne_prop_result_set *set)
     if (!ctx || !uri || !uri->path || !set)
         return;
 
-    dav_props *result = ne_calloc(sizeof(dav_props));
-    result->path = ne_path_unescape(uri->path);
+    char *tmp_path = (char *) ne_malloc(strlen(uri->path) + 1);
+    const char *from = uri->path;
 
 #endif /* NE_VERSION_MINOR >= 26 */
 
+    char *to = tmp_path;
+    while (*from) {
+        while (*from == '/' && *(from + 1) == '/')
+            from++;
+        *to++ = *from++;
+    }
+    *to = 0;
+    dav_props *result = ne_calloc(sizeof(dav_props));
+    result->path = ne_path_unescape(tmp_path);
+    free (tmp_path);
+
+#if NE_VERSION_MINOR < 26
+    ne_uri_free(&uri);
+#endif
+
+
     if (!result->path || strlen(result->path) < 1) {
         dav_delete_props(result);
         return;
@@ -1889,7 +1917,6 @@ ssl_verify(void *userdata, int failures, const ne_ssl_certificate *cert)
     char *issuer = ne_ssl_readable_dname(ne_ssl_cert_issuer(cert));
     char *subject = ne_ssl_readable_dname(ne_ssl_cert_subject(cert));
     char *digest = ne_calloc(NE_SSL_DIGESTLEN);
-    int ret = 0;
     if (!issuer || !subject || ne_ssl_cert_digest(cert, digest) != 0) {
         if (have_terminal) {
             error(0, 0, _("error processing server certificate"));
@@ -1897,9 +1924,13 @@ ssl_verify(void *userdata, int failures, const ne_ssl_certificate *cert)
             syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR),
                    _("error processing server certificate"));
         }
-        ret = -1;
+        if (issuer) free(issuer);
+        if (subject) free(subject);
+        if (digest) free(digest);
+        return -1;
     }
 
+    int ret = -1;
     if (have_terminal) {
         if (failures & NE_SSL_NOTYETVALID)
             error(0, 0, _("the server certificate is not yet valid"));
@@ -1919,7 +1950,6 @@ ssl_verify(void *userdata, int failures, const ne_ssl_certificate *cert)
         printf("\n");
         printf(_("  fingerprint: %s"), digest);
         printf("\n");
-        if (!ret) {
             printf(_("You only should accept this certificate, if you can\n"
                      "verify the fingerprint! The server might be faked\n"
                      "or there might be a man-in-the-middle-attack.\n"));
@@ -1930,10 +1960,9 @@ ssl_verify(void *userdata, int failures, const ne_ssl_certificate *cert)
             len = getline(&s, &n, stdin);
             if (len < 0)
                 abort();
-            if (rpmatch(s) < 1)
-                ret = -1;
+            if (rpmatch(s) > 0)
+                ret = 0;
             free(s);
-        }
     } 
 
     if (failures & NE_SSL_NOTYETVALID)
@@ -1959,9 +1988,9 @@ ssl_verify(void *userdata, int failures, const ne_ssl_certificate *cert)
         syslog(LOG_MAKEPRI(LOG_DAEMON, LOG_ERR), _("  accepted by user"));
     }
 
-    free(issuer);
-    free(subject);
-    free(digest);
+    if (issuer) free(issuer);
+    if (subject) free(subject);
+    if (digest) free(digest);
     return ret;
 }
 
